module Gitlab
  module Conflict
    class FileCollection
      class ConflictSideMissing < StandardError
      end

      attr_reader :merge_request, :our_commit, :their_commit

      def initialize(merge_request)
        @merge_request = merge_request
        @our_commit = merge_request.source_branch_head.raw.raw_commit
        @their_commit = merge_request.target_branch_head.raw.raw_commit
      end

      def repository
        merge_request.project.repository
      end

      def merge_index
        @merge_index ||= repository.rugged.merge_commits(our_commit, their_commit)
      end

      def resolve_conflicts!(params, user:)
        resolutions = params[:sections]
        commit_message = params[:commit_message] || default_commit_message
        rugged = repository.rugged
        committer = repository.user_to_committer(user)

        files.each do |file|
          file.resolve!(resolutions, index: merge_index, rugged: rugged)
        end

        new_tree = merge_index.write_tree(rugged)

        Rugged::Commit.create(rugged,
                              author: committer,
                              committer: committer,
                              tree: new_tree,
                              message: commit_message,
                              parents: [our_commit, their_commit].map(&:oid),
                              update_ref: Gitlab::Git::BRANCH_REF_PREFIX + merge_request.source_branch)
      end

      def files
        @files ||= merge_index.conflicts.map do |conflict|
          raise ConflictSideMissing unless conflict[:theirs] && conflict[:ours]

          Gitlab::Conflict::File.new(merge_index.merge_file(conflict[:ours][:path]),
                                     conflict,
                                     repository: repository)
        end
      end

      def as_json(opts = nil)
        {
          target_branch: merge_request.target_branch,
          source_branch: merge_request.source_branch,
          commit_sha: merge_request.diff_head_sha,
          commit_message: default_commit_message,
          files: files
        }
      end

      def default_commit_message
        conflict_filenames = merge_index.conflicts.map do |conflict|
          "#   #{conflict[:ours][:path]}"
        end

        <<EOM.chomp
Merge branch '#{merge_request.source_branch}' into '#{merge_request.target_branch}'

# Conflicts:
#{conflict_filenames.join("\n")}
EOM
      end
    end
  end
end
